//
//  AppDelegate.swift
//  MagicScoreRotate
//
//  Created by Matt on 12/8/16.
//  Copyright © 2016 Matt Rajca. All rights reserved.
//

import Cocoa
import QuartzCore

@NSApplicationMain
final class AppDelegate: NSObject, NSApplicationDelegate {
	@IBOutlet weak var window: NSWindow!

	func applicationDidFinishLaunching(_ aNotification: Notification) {
		let invention1URL = Bundle.main.urlForImageResource("bach-invention-01-a4.png")!
		let invention1 = CIImage(contentsOf: invention1URL)!

		let monochromeFilter = CIFilter(name: "CIPhotoEffectTonal")!
		monochromeFilter.setValue(invention1, forKey: kCIInputImageKey)

		let edgesFilter = CIFilter(name: "CIEdges")!
		edgesFilter.setValue(monochromeFilter.outputImage!, forKey: kCIInputImageKey)
		let outputImageUncropped = edgesFilter.outputImage!

		// Crop 1 pixel of white margin along the right and bottom edges.
		let outputImage = outputImageUncropped.cropping(to: CGRect(x: 1, y: 1, width: invention1.extent.width-2, height: invention1.extent.height-2))

		let imageRep = NSBitmapImageRep(ciImage: outputImage)
		var edgeMap = Array<Bool>(repeating: false, count: imageRep.pixelsHigh * imageRep.pixelsWide)

		for y in 0..<imageRep.pixelsHigh {
			for x in 0..<imageRep.pixelsWide {
				var buffer = Array<Int>(repeating: 0, count: 4 /* RGBA */)
				imageRep.getPixel(&buffer, atX: x, y: y)

				edgeMap[y * imageRep.pixelsWide + x] = buffer[0] > 127
			}
		}

		let thetaSubdivisions = 2
		let thetaMax = 45
		let thetaMin = -thetaMax
		let thetaCells = thetaMax * thetaSubdivisions * 2 /* plus/minus */ + 1

		let rFactor = 4
		let rMax = Int(sqrt(pow(Double(imageRep.pixelsWide), 2) + pow(Double(imageRep.pixelsHigh), 2)))
		let rCells = rMax / rFactor

		var cells = [Int](repeating: 0, count: thetaCells * rCells)

		for y in 0..<imageRep.pixelsHigh {
			for x in 0..<imageRep.pixelsWide {
				guard edgeMap[y * imageRep.pixelsWide + x] else {
					continue
				}

				for theta in stride(from: Double(thetaMin), through: Double(thetaMax), by: 0.5) {
					// r = x * cos(theta) + y * sin(theta)
					let r = Int(cos(theta * M_PI / 180.0) * Double(x) + sin(theta * M_PI / 180.0) * Double(y))

					if r < rMax && r >= 0 {
						let thetaIndex = Int(round(theta * Double(thetaSubdivisions))) + thetaMax * thetaSubdivisions
						cells[thetaIndex * rCells + r / rFactor] += 1
					}
				}
			}
		}

		typealias Cell = (frequency: Int, theta: Double)

		var cellTuples = [Cell]()
		for theta in 0..<thetaCells {
			for r in 0..<rCells {
				let frequency = cells[theta * rCells + r]
				let thetaScaled = Double(theta) / Double(thetaSubdivisions) - Double(thetaMax)
				cellTuples.append(Cell(frequency: frequency, theta: thetaScaled))
			}
		}

		cellTuples.sort { return $0.frequency > $1.frequency }
		let top5 = cellTuples.prefix(upTo: 5)
		let sum = top5.reduce(0.0) { $0 + $1.theta }
		print(sum / Double(top5.count))
	}
}
